﻿using System;
using System.Collections.Generic;
using System.Text;

namespace MapApp.Model
{
    public class HistoryFile
    {
        public delegate List<string> HistoryFileLoaderDelegate(string fileName);
        public delegate void HistoryFileSaverDelegate(List<string> contents, string fileName);
        public HistoryFileLoaderDelegate HistoryFileLoader;
        public HistoryFileSaverDelegate HistoryFileSaver;

        public List<String> Load(string fileName)
        {
            if (HistoryFileLoader == null)
            {
                throw new HistoryFileException("HistoryFileLoader delegate has not been set");
            }
            else
            {
                try
                {
                    return HistoryFileLoader(fileName);
                }
                catch (Exception)
                {
                    return new List<string>(); // The history file may not exist yet. Return an empty list
                }
            }
        }

        public void Save(List<string> contents, string fileName)
        {
            if (HistoryFileSaver == null)
            {
                throw new HistoryFileException("HistoryFileSaver delegate has not been set");
            }
            else
            {
                HistoryFileSaver(contents, fileName);
            }
        }
    }

    public class HistoryFileException : Exception
    {
        public HistoryFileException(string s) : base(s) { }
    }
}
