using System;
using System.Collections.Generic;
using System.Text;

using MapApp.Model;
using MapApp.Ui.Controls;

namespace MapApp.Model
{
    /// <summary>
    /// Abstraction of the currently active map. Static methods for things like
    ///  ZoomTo(MapLocation), and things like that.
    public class Map
    {
        public event MapLocationsChangedEventHandler MapLocationsChanged;
        public delegate void MapLocationsChangedEventHandler(object sender, EventArgs e);

        private Guid _ownerGuid;
        private MapState _mapState = new MapState();
        private MapLocationCollection _mapLocations = new MapLocationCollection();


        #region Properties

        public Guid OwnerGuid
        {
            get { return _ownerGuid; }
            set { _ownerGuid = value; }
        }

        public MapState MapState
        {
            get { return _mapState; }
            //set { _mapState = value; }
        }

        public MapLocationCollection MapLocations
        {
            get { return _mapLocations; }
            set
            {
                if (_mapLocations != value)
                {
                    _mapLocations = value;
                    if (_mapLocations != null)
                    {
                        _mapLocations.RaiseListChangedEvents = true;
                        _mapLocations.ListChanged += new System.ComponentModel.ListChangedEventHandler(_mapLocations_ListChanged);
                        if (MapLocationsChanged != null)
                        {
                            MapLocationsChanged(_mapLocations, new EventArgs());
                        }
                    }
                }
            }
        }

        void _mapLocations_ListChanged(object sender, System.ComponentModel.ListChangedEventArgs e)
        {
            if (MapLocationsChanged != null)
            {
                MapLocationsChanged(sender, e);
            }
        }

        #endregion

        #region Methods

        public void PanTo(MapLocation location)
        {
            AppUtil.frmMain.MapControl.PanTo(location);
        }

        public void ZoomTo(MapLocation location)
        {
            AppUtil.frmMain.MapControl.ZoomTo(location);
        }

        public void ZoomToAllMarkers()
        {
            AppUtil.frmMain.MapControl.ZoomToAllMarkers();
        }

        public void SetOverviewMapVisible(bool visible)
        {
            this.MapState.ShowOverviewMap = visible;
            AppUtil.frmMain.MapControl.SetOverviewMapVisible(visible);
        }

        public void SetScaleBarVisible(bool visible)
        {
            AppUtil.frmMain.MapControl.SetScaleBarVisible(visible);
        }

        public void NewMarker(MapLocation location)
        {
            location.Id = AppUtil.frmMain.MapControl.NewMarker(location);
            AppUtil.frmMain.MapControl.Map.MapLocations.Add(location);
        }

        #endregion
    }
}
