﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MapApp.Model;

namespace MapApp.Ui.Controls
{
    public partial class FrmMarkers : AppFormBase
    {
        private MapLocationCollection _mapLocations;
        private GoogleMapControl _mapControl;

        public MapLocationCollection MapLocations
        {
            get { return _mapLocations; }
            set
            {
                _mapLocations = value;
                mapLocationsBindingSource.DataSource = _mapLocations;
            }
        }

        public GoogleMapControl MapControl
        {
            get { return _mapControl; }
            set { _mapControl = value; }
        }

        public FrmMarkers()
        {
            InitializeComponent();
            CustomInitialize();
        }

        private void CustomInitialize()
        {
            mapLocationCollectionDataGridView.DataSource = mapLocationsBindingSource;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void chkShowAllMarkerLabels_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = sender as CheckBox;
            if (checkBox == null) return;
            if (MapControl == null) return;
            if (checkBox.CheckState == CheckState.Checked)
            {
                MapControl.ShowAllLabels();
            }
            else
            {
                MapControl.HideAllLabels();
            }
        }

        private void zoomToMarkerMenuItem_Click(object sender, EventArgs e)
        {
            DataGridViewRow currentRow = mapLocationCollectionDataGridView.CurrentRow;
            MapLocation loc = MapLocations[currentRow.Index];
            MapControl.ZoomTo(loc);
        }

        private void mapLocationCollectionDataGridView_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.ColumnIndex >= 0 && e.RowIndex >= 0)
            {
                mapLocationCollectionDataGridView.CurrentCell = mapLocationCollectionDataGridView.Rows[e.RowIndex].Cells[e.ColumnIndex];
            } else 
            if (e.ColumnIndex == -1 && e.RowIndex >= 0)
            {
                mapLocationCollectionDataGridView.CurrentCell = mapLocationCollectionDataGridView.Rows[e.RowIndex].Cells[0];
                mapLocationCollectionDataGridView.Rows[e.RowIndex].Selected = true;
            }
        }

        private void panToMarkerMenuItem_Click(object sender, EventArgs e)
        {
            DataGridViewRow currentRow = mapLocationCollectionDataGridView.CurrentRow;
            MapLocation loc = MapLocations[currentRow.Index];
            MapControl.PanTo(loc);
        }

    }
}
